//  RandNum.h
//  Written by:  Brian Lee Bowers
//  30 July 1998

//--  Inclusion Guard  --//
#ifndef RANDNUM_H_
#define RANDNUM_H_

//-  Include Header Files  -//
#include <time.h>

//==========================================================
//  Class:  RandGen
//  Purpose:  random number generator
//  Written by:  Brian Lee Bowers
//  Revisions:  0.0.1  30 July 1998
//
//  0.0.1  I have to admit, right up front, that this is
//    written to avoid using Scott Robert Ladd's class,
//    presented in "C++ Templates and Tools" for copyright
//    and royalty reasons.  The algorithm I use for pseudo-
//    random numbers is presented in Robert Sedgewick's
//    book "Algorithms in C++" and is based on the linear
//    congruential method introduced by D Lehmer in 1951.
//
//    To generate a pseudo-random number, use the following
//    algorithm:
//      a(1) = seed
//      a(n) = (a(n-1) * b + 1) % m
//    with the following restrictions on b and m
//      m -- should be large
//      b -- not too large and not too small
//             (one digit less than m??)
//           no pattern to digits, except it must end with
//             x21, where x is an even digit
//    Because of multiplying a(n-1) and b together, we will,
//    eventually, have to contend with overflow.
//    Dr. Sedgewick provides a clear way around the problem:
//      prod = ((m * b/m) + (b % m)) * ((m * (a(n-1)/m)) + (a(n-1) % m))
//    which provides the following result:
//      prod = m^2 * (a*b)/m + m * ((b/m * (a(n-1)%m)) + (a(n-1)/m * (b%m)))
//           + ((a%m) * b%m))
//    If m = 10^4, and we are only interested in the low
//    eight digits of the number (up to 99,999,999), we can
//    ignore the first term of the result above (m^2 will be
//    10^8 putting that portion out of the range we need
//    automatically) giving us the following equation:
//      prod = m * ((b/m * (a(n-1)%m)) + (a(n-1)/m * b%m)) + ((a%m) * (b%m))
//    Allowing b and a to be double word length (32 bits for
//    this example) allows both a and b to range up to a
//    value under 4,294,967,295 when divided by m will give
//    an integer value between 0 and 429,496 and when moduloed
//    by m will give a value between 0 and 9,999 inclusive.
//    Maximally, this could result in 429,496 being multiplied
//    by 9,999, which is 4,294,530,504.  I know of very few
//    instances where a full-range of four billion numbers
//    will be necessary.  Using m^2 as a modulus on the result
//    will provide values ranging from 0 to 99,999,999, which
//    even if doubled, will never overflow a 32-bit double
//    word.
//
//    Therefore, given a and b, of unknown values, and
//    m of some large value less then 2^16, the formula
//    for a pseudo-random number generator becomes:
//      a(n) = ((((m * ((b/m) * a(n-1)%m)) % m^2) *
//               ((m * ((a(n-1)/m) * b%m)) % m^2) +
//               ((a(n-1)%m) * (b%m))) + 1) % m
//    This assumes that we are limited to a and b being
//    not more than 32-bits.  It a larger numeric type is
//    available, m can increase in value accordingly.
//==========================================================

class RandNum
{
public:
  RandNum(unsigned long seed = time(0));
  ~RandNum();

  unsigned int operator ()(unsigned int range);

private:
  unsigned long a;
  unsigned long b;
  unsigned long m;
};

//--  End Inclusion Guard  --//
#endif

